#!/usr/bin/perl -w

# Be strict to avoid messy code
use strict;

# Fix latin characters
use Encode qw(encode decode);
my $enc = "utf-8";

# Use FindBin module to get script directory
use FindBin;

# Name of our xrc gui resource file
my $xrc_gui_file = "../GUI/logviewer.xrc";

# Disable buffering
$|=1;

# Get script directory
my $cwd = $FindBin::RealBin;
# Get script filename
my $scriptname = $FindBin::Script;
# Detect the current OS
my $OS = "$^O";

# If no arguments are passed
if ("@ARGV" eq '')
{
	# Set default as the parameter
	@ARGV = qw(default);
}

# Set this to 1 if you have compiled wxwidgets with cocoa support (lacks functions and stability)
my $use_cocoa = 0;

# If we are on macosx
if ($OS =~ /darwin/)
{
	# If $use_cocoa environment variable contains cocoa
	if ($ENV{"use_cocoa"} =~ /cocoa/)
	{
		# Set $use_cocoa to 1
		$use_cocoa = 1;
	}

	# If we are on mac use a special xrc file instead
	if ($use_cocoa =~ /1/)
	{
		# Use alternative gui file
		$xrc_gui_file = "../GUI/logviewer_cocoa.xrc";
	}
}

# Make a variable for users homedir
my $HOME;
# If we are on windows
if ($OS =~ /MSWin32/)
{
	# Get the environment variable for USERPROFILE
	$HOME = $ENV{"USERPROFILE"};
	# Replace all / with \
	$HOME =~ s/\//\\/g;
	
	require Win32::Console;

	# you may want to use the non-instanciated form to avoid confuzion :)
	my $codepage = Win32::Console::OutputCP();
	Win32::Console::OutputCP(1252);
}
# Else we are on UNIX
else
{
	$HOME = $ENV{"HOME"};
}

# Load the settings
require "$cwd/../Modules/settings.pm";
# Get the settings
my $settings = settings::get_settings();

# If the location contains ".app" then
if ($settings->{logroot} =~ m/(.+)\/.+\.app/)
{
	my $osxpath = $1;
	# Move outside the .app folder (osx application bundle)
	$settings->{logroot} =~ s/$cwd/$osxpath/;
}
else
{
	# Use regex to substitute $cwd with $cwd/.. (if nothing matches it will do nothing)
	$settings->{logroot} =~ s/$cwd/$cwd\/\.\./;
}


# Define a text inside the script for use
my $helptext = << "helptext";
How to use the search:

To do a simple search just put the search
text inside quotes like this.
"Text to search".

To use Advanced REGEX Search please use the
perl regex syntax.
Here is an example to search for Windows
or Linux: 
(windows|linux)
helptext

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package wxTopLevelFrame;

use Wx qw[:everything];
use Wx::XRC;

# Use Encode module to fix latin characters
use Encode qw(encode decode);

# Which events shall we include
use Wx::Calendar;
use Wx::Event qw(EVT_BUTTON EVT_LISTBOX_DCLICK EVT_CALENDAR_SEL_CHANGED);

# Use File::Path to get some file modules
use File::Path qw(make_path);

use base qw(Wx::Frame);

sub new
{
	# Create a class
	my $class = shift;
	
	# Assign class object to $self
	my $self = $class->SUPER::new;
	
	# Initialize everything
	$self->initialize;
	
	return $self;
}

sub initialize
{
	# Get pointers
	my $self = shift;
	
	# Create mutators for widgets (enter the objectname for every object here)
	$self->create_mutator
	(
		qw
		(
			xrc_resource
			viewinglog
			logview
			calendarlog
			searchpattern
			fromdate
			todate
			loglist
			close
			save
			search
			tabwindow
			search_help
			lbl_logdate
			calendarpicker
			maclogs
		)
	);
	
	load_xrc_gui($self);
	
	set_events($self);
	
	set_tooltips($self);
	
	# Load the current log
	cal_changed($self);
	
}

sub load_xrc_gui
{
	# Get the pointers
	my $self = shift;
	
	# Get the xrc file
	my $xrc_file = "$cwd/$xrc_gui_file";
	
	# Initialize WX
	Wx::InitAllImageHandlers();
	
	# Create xrc/xml resource
	$self->xrc_resource = Wx::XmlResource->new;
	# Initialize handlers
	$self->xrc_resource->InitAllHandlers;
	# Load the xrc file
	$self->xrc_resource->Load($xrc_file);
	
	# Tell what window/frame to load
	$self->xrc_resource->LoadFrame($self,undef,"mainwindow");
}

#
#---------------------------------------- *** ----------------------------------------
#

sub set_events
{
	# Get the pointers
	my $self = shift;
	
	# Setup the events
	# EVT_BUTTON($self, Wx::XmlResource::GetXRCID('objectname'), \&function);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('save'), \&save_clicked);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('close'), \&close_clicked);
	EVT_BUTTON($self, Wx::XmlResource::GetXRCID('search'), \&search_clicked);
	EVT_LISTBOX_DCLICK($self, Wx::XmlResource::GetXRCID('loglist'), \&loglist_dclicked);
	
	# If we are on macosx and want to use the cocoa toolkit
	if ($use_cocoa =~ /1/)
	{
		# Define calendar changed event
		EVT_CALENDAR_SEL_CHANGED($self, Wx::XmlResource::GetXRCID('calendarpicker'), \&cal_changed);
		
		# Define a listbox doubleclicked event
		EVT_LISTBOX_DCLICK($self, Wx::XmlResource::GetXRCID('maclogs'), \&maclogs_dclicked);
		
		# Define Mac spesific widgets
		$self->calendarpicker = $self->FindWindow('calendarpicker');
		$self->maclogs = $self->FindWindow('maclogs');
	}
	# If we are on anything else
	else
	{
		# Define calendar changed event
		EVT_CALENDAR_SEL_CHANGED($self, Wx::XmlResource::GetXRCID('calendarlog'), \&cal_changed);
		
		# Define the calendar used select logs
		$self->calendarlog = $self->FindWindow('calendarlog');
	}
	
	# Find the widgets
	# $self->objectname = $self->FindWindow('objectname');
	$self->close = $self->FindWindow('close');
	$self->save = $self->FindWindow('save');
	$self->loglist = $self->FindWindow('loglist');
	$self->logview = $self->FindWindow('logview');
	$self->viewinglog = $self->FindWindow('viewinglog');
	$self->searchpattern = $self->FindWindow('searchpattern');
	$self->todate = $self->FindWindow('todate');
	$self->fromdate = $self->FindWindow('fromdate');
	$self->search = $self->FindWindow('search');
	$self->tabwindow = $self->FindWindow('tabwindow');
	$self->search_help = $self->FindWindow('search_help');
	$self->lbl_logdate = $self->FindWindow('lbl_logdate');
	
	# Change tab to the log viewer
	$self->tabwindow->ChangeSelection(0);
	
	# If we are on macosx
	if ($use_cocoa =~ /1/)
	{
		# Set the window size
		$self->SetSize(620,450);
		
		# Change widget sizes
		$self->loglist->SetSize(130,$self->loglist->GetSize->GetHeight());
		$self->maclogs->SetSize(130,$self->maclogs->GetSize->GetHeight());
	}
	# Else if we are on anything else
	else
	{
		# Set the window size
		$self->SetSize(620,450);
	}
	
	# Set minimum sizes
	$self->SetMinSize($self->GetSize);

	$self->search_help->SetLabel($helptext);
	my $w = $self->search_help->GetSize->GetWidth();
	$self->search_help->Wrap($w);

	# Set the proper title
	$self->SetTitle("LogViewer: @ARGV");
	
	# Define the attributes for the fromdate widget
	my $default_fromdate = Wx::DateTime->newFromDMY(31,9,2009);
	
	# Make sure that the fromdate widget date is the same date this
	# Project started
	$self->fromdate->SetValue($default_fromdate);
	
	# Set the window icon
	my $window_icon = Wx::Icon->new;
	$window_icon->LoadFile("$cwd/../GUI/win-ico.ico", wxBITMAP_TYPE_ICO);
	$self->SetIcon($window_icon);
}

#
#---------------------------------------- *** ----------------------------------------
#



# Create mutator function from "Programming Perl"
sub create_mutator
{

	my $self = shift;

	# From "Programming Perl" 3rd Ed. p338.
	for my $attribute (@_)
	{

		no strict "refs"; # So symbolic ref to typeglob works.
		no warnings;      # Suppress "subroutine redefined" warning.

		*$attribute = sub : lvalue
		{

			my $self = shift;

			$self->{$attribute} = shift if @_;
			$self->{$attribute};

		};

	}

}


### Events
sub cal_changed
{
	my ($self,$event) = @_;
	
	# Empty the textfield
	$self->logview->SetValue("");
	
	# Make variables to contain the dates
	my $year;
	my $month;
	my $day;
	
	# If we are on macosx and want to use the cocoa toolkit
	if ($use_cocoa =~ /1/)
	{
		# Get the selected date
		$year = $self->calendarpicker->GetValue->GetYear;
		$month = $self->calendarpicker->GetValue->GetMonth +1;
		$day = $self->calendarpicker->GetValue->GetDay;
	}
	# If we are on anything else
	else
	{
		# Get the selected date
		$year = $self->calendarlog->GetDate->GetYear;
		$month = $self->calendarlog->GetDate->GetMonth +1;
		$day = $self->calendarlog->GetDate->GetDay;
	}
	
	
	# If $month is lower than 10
	if ($month < 10)
	{
		# Append a zero infront of it
		$month = "0$month";
	}
	# If $day is lower than 10
	if ($day < 10)
	{
		# Append a zero infront of it
		$day = "0$day";
	}
	
	# Read the selected log
	my $logcontent = main::ReadFile($settings->{logroot}."/@ARGV/$year/$month/$year.$month.$day.txt");
	
	# If no errors from reading the file
	if ($logcontent !~ /error reading file/)
	{
		# Transfer the array pointer to a string
		my $logcontent = "@$logcontent";
		# Replace all newlines followed by a space with just a newline
		$logcontent =~ s/(\n|\r|\r\n|\n\r) /\n/g;
		
		# Put the log content into the logview
		$self->logview->SetValue("$logcontent");
		
	}
	# Set the label for which log we are viewing
	$self->lbl_logdate->SetLabel("Viewing: $year.$month.$day.txt");
	
	# If we are on macosx and want to use the cocoa toolkit
	if ($use_cocoa =~ /1/)
	{
		format_maclogs($self);
	}
	# If we are on anything else
	else
	{
		# Format the calendar to mark all dates with logs
		format_calendar($self);
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub format_maclogs
{
	my $self = shift;
	
	# Clear the list first
	$self->maclogs->Clear();
	
	# Get the dates
	my $from_month = $self->calendarpicker->GetValue->GetMonth +1;
	my $from_year = $self->calendarpicker->GetValue->GetYear;
	my $to_month = $self->calendarpicker->GetValue->GetMonth +1;
	my $to_year = $self->calendarpicker->GetValue->GetYear;
	
	# Make a temp variable for leading zero month
	my $temp_month;
	
	# Make temp variables for days
	my $day;
	my $temp_day;
	
	# Loop through the years
	for ($from_year = $from_year ; $from_year <= $to_year; $from_year ++)
	{
		# Loop through the months
		for ($from_month = $from_month; $from_month <= $to_month; $from_month ++)
		{
			# If $from_month contains only 1 digit
			if ($from_month < 10)
			{
				# Append zero infront of it and put into temp string
				$temp_month = "0$from_month";
			}
			else
			{
				# Put month into temp string
				$temp_month = "$from_month";
			}
			
			# Loop through the days
			for ($day = 1; $day <= 31; $day++)
			{
				# If $day contains only 1 digit
				if ($day < 10)
				{
					# Append zero infront of it
					$temp_day = "0$day";
				}
				else
				{
					# Transfer value to a string
					$temp_day = "$day";
				}
				
				# Check the file
				my $check = main::ReadFile($settings->{"logroot"}."/@ARGV/$from_year/$temp_month/$from_year.$temp_month.$temp_day.txt");

				# If the file could be opened
				if ($check !~ /error reading file/)
				{
					# Transfer the array pointer to a string
					$check = "@$check";
					
					# If filecontents contains something from the searchpattern
					if ($check =~ /.*/i)
					{
						# Add the filename to the loglist
						$self->maclogs->Append("$from_year.$temp_month.$temp_day");
					}
				}
			}
		}
		# We finished searching through a year so now we reset the months
		$from_month = 1;
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub loglist_dclicked
{
	my $self = shift;
	
	# Get the date that was doubleclicked
	my $log = $self->loglist->GetStringSelection();
	
	# Split the date by .
	# 0 = year
	# 1 = month
	# 2 = day
	my @logdate = split /\./,$log;
	
	# Define the attributes
	my $date = Wx::DateTime->new;
	$date->SetYear($logdate[0]);
	$date->SetMonth($logdate[1]-1);
	$date->SetDay($logdate[2]);
	
	# If we are on macosx and want to use the cocoa toolkit
	if ($use_cocoa =~ /1/)
	{
		# Set the date in the calendar
		$self->calendarpicker->SetValue($date);
	}
	# If we are on anything else
	else
	{
		# Set the date in the calendar
		$self->calendarlog->SetDate($date);
	}
	
	# Trigger the calendar changed event
	cal_changed($self);
	
	# Change tab to the log viewer
	$self->tabwindow->ChangeSelection(0);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub maclogs_dclicked
{
	my $self = shift;
	
	# Get the date that was doubleclicked
	my $log = $self->maclogs->GetStringSelection();
	
	# Split the date by .
	# 0 = year
	# 1 = month
	# 2 = day
	my @logdate = split /\./,$log;
	
	# Define the attributes
	my $date = Wx::DateTime->new;
	$date->SetYear($logdate[0]);
	$date->SetMonth($logdate[1]-1);
	$date->SetDay($logdate[2]);
	
	# Set the date in the calendar
	$self->calendarpicker->SetValue($date);
	
	# Trigger the calendar changed event
	cal_changed($self);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub search_clicked
{
	my $self = shift;
	
	# Clear the list first
	$self->loglist->Clear();
	
	# Get the search pattern
	my $searchpattern = $self->searchpattern->GetValue;
	
	# Get the dates
	my $from_month = $self->fromdate->GetValue->GetMonth +1;
	my $from_year = $self->fromdate->GetValue->GetYear;
	my $to_month = $self->todate->GetValue->GetMonth +1;
	my $to_year = $self->todate->GetValue->GetYear;
	
	# Make a temp variable for leading zero month
	my $temp_month;
	
	# Make temp variables for days
	my $day;
	my $temp_day;
	
	# Loop through the years
	for ($from_year = $from_year ; $from_year <= $to_year; $from_year ++)
	{
		# Loop through the months
		for ($from_month = $from_month; $from_month <= $to_month; $from_month ++)
		{
			# If $from_month contains only 1 digit
			if ($from_month < 10)
			{
				# Append zero infront of it and put into temp string
				$temp_month = "0$from_month";
			}
			else
			{
				# Put month into temp string
				$temp_month = "$from_month";
			}
			
			# Loop through the days
			for ($day = 1; $day <= 31; $day++)
			{
				# If $day contains only 1 digit
				if ($day < 10)
				{
					# Append zero infront of it
					$temp_day = "0$day";
				}
				else
				{
					# Transfer value to a string
					$temp_day = "$day";
				}
				
				# Check the file
				my $check = main::ReadFile($settings->{"logroot"}."/@ARGV/$from_year/$temp_month/$from_year.$temp_month.$temp_day.txt");

				# If the file could be opened
				if ($check !~ /error reading file/)
				{
					# Transfer the array pointer to a string
					$check = "@$check";
					
					# If we are not on windows
					if ($OS !~ /MSWin32/)
					{
						# Encode to utf-8
						$searchpattern = encode($enc,$searchpattern);
					}
					
					# If $searchpattern is quoted
					if ($searchpattern =~ /^".+"$/)
					{
						# Remove the quotes
						$searchpattern =~ s/(^"|"$)//g;
						
						# And then use quotemeta so that the $searchpattern
						# Is treated as a string in the regular expression
						$searchpattern = quotemeta($searchpattern);
					}
					# Else if $searchpattern starts with + or *
					elsif($searchpattern =~ /^(\*|\+|\?)/)
					{
						# Correct the regex so that we dont crash
						$searchpattern = ".$searchpattern";
					}
					
					# If filecontents contains something from the searchpattern
					if ($check =~ /$searchpattern/i)
					{
						# Add the filename to the loglist
						$self->loglist->Append("$from_year.$temp_month.$temp_day");
					}
					
				}
				
			}
			
			
		}
		# We finished searching through a year so now we reset the months
		$from_month = 1;
	}
}

#
#---------------------------------------- *** ----------------------------------------
#

sub format_calendar
{
	my $self = shift;
	
	# Make a for loop to reset all calendar attributes
	my $counter;
	for ($counter = 1; $counter <= 31; $counter++)
	{
		# Reset the attributes in the calendar
		$self->calendarlog->ResetAttr($counter);
	}
	
	# Get the selected date
	my $year = $self->calendarlog->GetDate->GetYear;
	my $month = $self->calendarlog->GetDate->GetMonth +1;
	
	# If $month contains only 1 digit then
	if ($month < 10)
	{
		# Append a zero infront of it
		$month = "0$month";
	}
	
	# Define font attributes
	my $font = Wx::Font->new($self);
	#$font->SetWeight(wxFONTWEIGHT_NORMAL);
	
	# If we are not on windows
	if ($OS !~ /(MSWin32|darwin)/)
	{
		# Make sure the font size stays the same
		$font->SetPointSize(10);
	}
	
	# Define the attributes
	my $attr = Wx::CalendarDateAttr->new;
	#$attr->SetTextColour( wxBLUE);
	$attr->SetBorderColour( wxBLUE );
	#$attr->SetBackgroundColour(wxLIGHT_GREY);
	$attr->SetBorder( wxCAL_BORDER_SQUARE );
	$attr->SetFont($font);
	
	my $day;
	# Loop through the days
	my $temp_day;
	for ($day = 1; $day <= 31; $day++)
	{
		# If $day contains only 1 digit then
		if ($day < 10)
		{
			# Append a zero infront of it
			$temp_day = "0$day";
		}
		else
		{
			# Transfer to another string
			$temp_day = "$day";
		}
		
		# Check the file
		my $check = main::ReadFile($settings->{"logroot"}."/@ARGV/$year/$month/$year.$month.$temp_day.txt");
		
		# if the file opened fine then
		if ($check !~ /error reading file/)
		{
			# Set attributes to the day containing a log
			$self->calendarlog->SetAttr($day,$attr);
		}		
	}
	
	# Reset the attributes in the calendar
	$self->calendarlog->EnableHolidayDisplay(1);
}

#
#---------------------------------------- *** ----------------------------------------
#



sub save_clicked
{
	my $self = shift;
	
	# Get the contents of the log
	my $logcontent = $self->logview->GetValue;
	
	# Make variables to contain the dates
	my $year;
	my $month;
	my $day;
	
	# If we are on macosx and want to use the cocoa toolkit
	if ($use_cocoa =~ /1/)
	{
		# Get the selected date
		$year = $self->calendarpicker->GetValue->GetYear;
		$month = $self->calendarpicker->GetValue->GetMonth +1;
		$day = $self->calendarpicker->GetValue->GetDay;
	}
	# If we are on anything else
	else
	{
		# Get the selected date
		$year = $self->calendarlog->GetDate->GetYear;
		$month = $self->calendarlog->GetDate->GetMonth +1;
		$day = $self->calendarlog->GetDate->GetDay;
	}
	
	# If $month contains only 1 digit then
	if ($month < 10)
	{
		# Append a zero infront of it
		$month = "0$month";
	}
	# If $day contains only 1 digit then
	if ($day < 10)
	{
		# Append a zero infront of it
		$day = "0$day";
	}
	
	# Create the folderpath to the log
	make_path($settings->{"logroot"}."/@ARGV/$year/$month");
	
	# Write it to the file
	main::WriteFile("$logcontent", ">", $settings->{"logroot"}."/@ARGV/$year/$month/$year.$month.$day.txt");
	
	# Display a messagebox to notify that the function is done
	Wx::MessageBox("Changes have now been saved.\nLogfile: $year.$month.$day.txt\nLogbook: @ARGV", "Changes Saved!", wxOK,$self);
}

#
#---------------------------------------- *** ----------------------------------------
#

sub close_clicked
{
	# Get pointers
	my ($self, $event) = @_;
	
	# Close window
	$self->Destroy();
}

#
#---------------------------------------- *** ----------------------------------------
#

sub set_tooltips
{
	my $self = shift;
		
	# Set tooltips with info about the settings
	# $self->objectname->SetToolTip("message");
	
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------

package application;
use base qw(Wx::App);

sub OnInit
{
	# Get pointers
	my $self = shift;
	
	# Create mainwindow(new window)
	my $mainwindow = wxTopLevelFrame->new(undef, -1);
	
	# Set mainwindo/topwindow
	$self->SetTopWindow($mainwindow);
	
	# Show the window
	$mainwindow->Show(1);
}

#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------
#---------------------------------------- *** ----------------------------------------


package main;

my $app = application->new;
$app->MainLoop;

# Read contents from a file and put it into a pointer
sub ReadFile 
{
	# Gets passed data from the function call
	my ($filename) = @_;

	# Makes an array to keep the inputdata
	my @inputdata;

	# Opens the passed file, if error it dies with the message "Can't open filename"
	open (my $FILE, "$filename") || return "error reading file";

	# While there is something in the file
	while(<$FILE>)
	{
		if ($OS =~ /MSWin32/)
		{
			# Push data into the inputdata array
			push(@inputdata, $_);
		}
		else
		{
			# Push data into the inputdata array
			push(@inputdata, encode($enc,$_));
		}
		
	}

	# Close the file
	close($FILE);

	# Return the pointer to the datafile inputdata
	return(\@inputdata);
}

#
#---------------------------------------- *** ----------------------------------------
#

# Write a file from scratch(deletes previous content)
sub WriteFile
{
	# Get the passed variables
	my ($content, $writemode, $outfile) = @_;
	
	# Open the outfile for Writing/Rewrite
	open (my $FILE, "$writemode$outfile");

	# Write the content passed to the function to the file
	print $FILE "$content";
	
	close ($FILE);
}

#
#---------------------------------------- *** ----------------------------------------
#

